@@ -771,12 +771,12 @@ static int daemonize(int nochdir, int noclose)
 	return 0;
 }
 
-static RETSIGTYPE handle_sigterm(int signum)
+static void handle_sigterm(int signum)
 {
 	siglongjmp(nilfs_cleanerd_env, 1);
 }
 
-static RETSIGTYPE handle_sighup(int signum)
+static void handle_sighup(int signum)
 {
 	nilfs_cleanerd_reload_config = 1;
 }
