@@ -5,7 +5,7 @@
  * can be found in COPYING file of the nilfs-utils package.
  *
  * Copyright (C) 2011-2012 Nippon Telegraph and Telephone Corporation.
- * Written by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp>
+ * Written by Ryusuke Konishi <konishi.ryusuke@gmail.com>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,15 +60,14 @@
 #include <errno.h>
 #include <signal.h>
 #include "nilfs.h"
-#include "vector.h"
+#include "util.h"
 #include "nilfs_gc.h"
-#include "cnoconv.h"
 
 extern int check_mount(const char *device);
 
 #ifdef _GNU_SOURCE
 #include <getopt.h>
-const static struct option long_option[] = {
+static const struct option long_option[] = {
 	{"help", no_argument, NULL, 'h'},
 	{"verbose", no_argument, NULL, 'v'},
 	{"yes", no_argument, NULL, 'y'},
@@ -91,37 +90,19 @@ const static struct option long_option[] = {
 
 /* get device size through ioctl */
 #ifndef BLKGETSIZE64
-#define BLKGETSIZE64 	_IOR(0x12, 114, size_t)
+#define BLKGETSIZE64	_IOR(0x12, 114, size_t)
 #endif
 
 #ifndef FIFREEZE
-#define FIFREEZE 	_IOWR('X', 119, int)
+#define FIFREEZE	_IOWR('X', 119, int)
 #define FITHAW		_IOWR('X', 120, int)
 #endif
 
-/* general macros */
-#ifndef DIV_ROUND_UP
-#define DIV_ROUND_UP(n,m)   (((n) + (m) - 1) / (m))
-#endif
-
-#ifndef ARRAY_SIZE
-#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
-#endif
-
-#ifndef min_t
-#define min_t(type,x,y) \
-	({ type __x = (x); type __y = (y); __x < __y ? __x : __y; })
-#define max_t(type,x,y) \
-	({ type __x = (x); type __y = (y); __x > __y ? __x : __y; })
-#endif
-
-#define nilfs_cnt64_ge(a, b)	((__s64)(a) - (__s64)(b) >= 0)
-
 /* options */
 static char *progname;
-static int show_version_only = 0;
-static int verbose = 0;
-static int assume_yes = 0;
+static int show_version_only;
+static int verbose;
+static int assume_yes;
 static int show_progress = 1;
 
 /* global variables */
@@ -157,8 +138,8 @@ static int pm_labelwidth = 10;
 static __u64 pm_max;
 static __u64 pm_done;
 static int pm_curpos;
-static int pm_in_progress = 0;  /* 0: off, 1: on, -1: interrupted */
-const static char *pm_label;
+static int pm_in_progress;	/* 0: off, 1: on, -1: interrupted */
+static const char *pm_label;
 
 static void nilfs_resize_logger(int priority, const char *fmt, ...)
 {
@@ -314,7 +295,7 @@ static int nilfs_resize_segment_is_protected(struct nilfs *nilfs, __u64 segnum)
 		return -1;
 	}
 	segseq = nilfs_get_segment_seqnum(nilfs, segment, segnum);
-	if (nilfs_cnt64_ge(segseq, protseq))
+	if (cnt64_ge(segseq, protseq))
 		ret = 1;
 	nilfs_put_segment(nilfs, segment);
 	return ret;
@@ -366,17 +347,16 @@ static int nilfs_resize_check_free_space(struct nilfs *nilfs, __u64 newnsegs)
 		nsegs = (sustat.ss_nsegs - newnsegs) - sustat.ss_ncleansegs
 			+ nrsvsegs;
 		nbytes  = (nsegs * blocks_per_segment) << blocksize_bits;
-		myprintf("Error: the filesystem does not have enough free "
-			 "space.\n"
-			 "       At least %llu more segments (%llu bytes) "
-			 "are required.\n", nsegs, nbytes);
+		myprintf("Error: the filesystem does not have enough free space.\n"
+			 "       At least %llu more segments (%llu bytes) are required.\n",
+			 nsegs, nbytes);
 		return -1;
 	} else if (verbose) {
 		nsegs = sustat.ss_ncleansegs - (sustat.ss_nsegs - newnsegs)
 			- nrsvsegs;
 		nbytes  = (nsegs * blocks_per_segment) << blocksize_bits;
-		myprintf("%llu free segments (%llu bytes) will be left "
-			 "after shrinkage.\n", nsegs, nbytes);
+		myprintf("%llu free segments (%llu bytes) will be left after shrinkage.\n",
+			 nsegs, nbytes);
 	}
 	return 0;
 }
@@ -406,8 +386,8 @@ nilfs_resize_find_movable_segments(struct nilfs *nilfs, __u64 start,
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
-				 "movable segments: %s\n", strerror(errno));
+			myprintf("Error: operation failed during searching movable segments: %s\n",
+				 strerror(errno));
 			return -1;
 		}
 		for (i = 0; i < nsi; i++, segnum++) {
@@ -422,9 +402,10 @@ nilfs_resize_find_movable_segments(struct nilfs *nilfs, __u64 start,
 			rest--;
 		}
 	}
-	return (snp - segnumv); /* return the number of found segments */
+	return snp - segnumv; /* return the number of found segments */
 }
 
+#if 0
 static int
 nilfs_resize_get_latest_segment(struct nilfs *nilfs, __u64 start, __u64 end,
 				__u64 *segnump)
@@ -442,8 +423,8 @@ nilfs_resize_get_latest_segment(struct nilfs *nilfs, __u64 start, __u64 end,
 			      NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
-				 "latest segment: %s\n", strerror(errno));
+			myprintf("Error: operation failed during searching latest segment: %s\n",
+				 strerror(errno));
 			return -1;
 		}
 		assert(nsi > 0);
@@ -462,6 +443,7 @@ nilfs_resize_get_latest_segment(struct nilfs *nilfs, __u64 start, __u64 end,
 	}
 	return ret;
 }
+#endif
 
 static ssize_t
 nilfs_resize_find_active_segments(struct nilfs *nilfs, __u64 start, __u64 end,
@@ -479,8 +461,8 @@ nilfs_resize_find_active_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
-				 "active segments: %s\n", strerror(errno));
+			myprintf("Error: operation failed during searching active segments: %s\n",
+				 strerror(errno));
 			return -1;
 		}
 		for (i = 0; i < nsi; i++, segnum++) {
@@ -491,7 +473,7 @@ nilfs_resize_find_active_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 			}
 		}
 	}
-	return (snp - segnumv); /* return the number of found segments */
+	return snp - segnumv; /* return the number of found segments */
 }
 
 static ssize_t
@@ -510,8 +492,8 @@ nilfs_resize_find_inuse_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
-				 "in-use segments: %s\n", strerror(errno));
+			myprintf("Error: operation failed during searching in-use segments: %s\n",
+				 strerror(errno));
 			return -1;
 		}
 		for (i = 0; i < nsi; i++, segnum++) {
@@ -521,7 +503,7 @@ nilfs_resize_find_inuse_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 			}
 		}
 	}
-	return (snp - segnumv); /* return the number of found segments */
+	return snp - segnumv; /* return the number of found segments */
 }
 
 static ssize_t
@@ -540,8 +522,8 @@ nilfs_resize_count_inuse_segments(struct nilfs *nilfs, __u64 start, __u64 end)
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during counting "
-				 "in-use segments: %s\n", strerror(errno));
+			myprintf("Error: operation failed during counting in-use segments: %s\n",
+				 strerror(errno));
 			return -1;
 		}
 		for (i = 0; i < nsi; i++, segnum++) {
@@ -586,8 +568,6 @@ static ssize_t nilfs_resize_move_segments(struct nilfs *nilfs,
 	int rv = 0;
 	int ret;
 
-	sync();
-
 	for (snp = segnumv; rest > 0; snp += nc, rest -= nc) {
 		nc = min_t(unsigned long, rest, nsegments_per_clean);
 
@@ -621,31 +601,48 @@ static ssize_t nilfs_resize_move_segments(struct nilfs *nilfs,
 	return nmoved;
 }
 
-static int nilfs_resize_try_update_log_cursor(struct nilfs *nilfs)
+static int __nilfs_resize_try_update_log_cursor(struct nilfs *nilfs)
 {
+	nilfs_cno_t cno;
 	int arg = 0;
 	int ret = -1;
 
+	nilfs_sync(nilfs, &cno);
+
 	if (ioctl(nilfs->n_iocfd, FIFREEZE, &arg) == 0 &&
 	    ioctl(nilfs->n_iocfd, FITHAW, &arg) == 0)
 		ret = 0;
 
-	sync();
 	nilfs_resize_update_sustat(nilfs);
 
 	return ret;
 }
 
+static int nilfs_resize_try_update_log_cursor(struct nilfs *nilfs,
+					      const char *reason)
+{
+	int ret;
+
+	if (verbose)
+		myprintf("%s.\nTrying to update log cursor ..", reason);
+
+	ret = __nilfs_resize_try_update_log_cursor(nilfs);
+
+	if (verbose)
+		myprintf(ret == 0 ? " ok.\n" : " failed.\n");
+	return ret;
+}
+
 static int nilfs_resize_reclaim_nibble(struct nilfs *nilfs,
 				       unsigned long long start,
 				       unsigned long long end,
-				       unsigned long count)
+				       unsigned long count, int unprotect)
 {
 	__u64 segnumv[2], segnum;
 	ssize_t nfound, nmoved = 0;
 	unsigned long nc;
 	unsigned long long end2 = end;
-	int freeze_thaw = 0;
+	int log_cursor_updated = 0;
 	int ret;
 
 	segnum = start;
@@ -667,10 +664,14 @@ retry:
 		if (nm < 0)
 			goto failed;
 
-		sync();
 		nmoved += nm;
-		if (nmoved >= count)
+		if (nmoved >= count) {
+			if (unprotect && !log_cursor_updated) {
+				nilfs_resize_try_update_log_cursor(
+					nilfs, "Hit protected segment");
+			}
 			return 0;
+		}
 	}
 
 	if (end >= start) {
@@ -678,23 +679,15 @@ retry:
 		end = start - 1;
 		goto retry;
 	}
-	if (!freeze_thaw) {
-		if (verbose)
-			myprintf("No movable segment.\n"
-				 "Trying to update log cursor to make movable "
-				 "segments ..");
-
-		ret = nilfs_resize_try_update_log_cursor(nilfs);
+	if (!log_cursor_updated) {
+		ret = nilfs_resize_try_update_log_cursor(
+			nilfs, "No movable segment");
 		if (!ret) {
-			if (verbose)
-				myprintf(" ok.\n");
 			segnum = start;
 			end = end2;
-			freeze_thaw = 1;
+			log_cursor_updated = 1;
 			goto retry;
 		}
-		if (verbose)
-			myprintf(" failed.\n");
 	}
 	myprintf("Error: couldn't move any segments.\n");
 	return -1;
@@ -708,91 +701,50 @@ static int nilfs_resize_move_out_active_segments(struct nilfs *nilfs,
 						 unsigned long long start,
 						 unsigned long long end)
 {
-	const static struct timespec retry_interval = { 0, 500000000 };
+	static const struct timespec retry_interval = { 0, 500000000 };
 							/* 500 msec */
-	__u64 segnum;
 	ssize_t nfound;
-	int latest = 0;
 	int retrycnt = 0;
-	int ret;
 
-retry:
-	if (nilfs_resize_update_sustat(nilfs) < 0)
-		return -1;
+	while (1) {
+		if (nilfs_resize_update_sustat(nilfs) < 0)
+			return -1;
 
-	nfound = nilfs_resize_find_active_segments(
-		nilfs, start, end, segnums, NILFS_RESIZE_NSEGNUMS);
-	if (nfound < 0)
-		return -1;
+		nfound = nilfs_resize_find_active_segments(
+			nilfs, start, end, segnums, NILFS_RESIZE_NSEGNUMS);
+		if (nfound < 0)
+			return -1;
+		if (!nfound)
+			break;
 
-	if (!nfound) {
-		if (latest) {
-			/* reset the latest segment info if it was moved */
-			ret = nilfs_resize_get_latest_segment(
-				nilfs, segnum, segnum, &segnum);
-			if (ret < 0)
-				return -1;
-			else if (!ret)
-				latest = 0;
-		}
-		if (!latest) {
-			/* get the latest segment info */
-			ret = nilfs_resize_get_latest_segment(
-				nilfs, start, end, &segnum);
-			if (ret < 0)
-				return -1;
-			else if (ret)
-				latest = 1;
-		}
-		if (latest) {
-			/* test if the latest segment is protected or not */
-			ret = nilfs_resize_segment_is_protected(nilfs, segnum);
-			if (ret < 0)
-				return -1;
-			else if (ret)
-				nfound = 2;
-		}
-	}
-
-	if (nfound > 0) {
 		if (retrycnt >= 6) {
-			myprintf("Error: Failed to move active segments -- "
-				 "give up.\n");
+			myprintf("Error: Failed to move active segments -- give up.\n");
 			return -1;
 		}
 		if (verbose && !retrycnt) {
 			myprintf("Active segments are found in the range.\n"
 				 "Trying to move them.\n");
 		}
-		if (nilfs_resize_reclaim_nibble(nilfs, start, end, nfound) < 0)
+		if (nilfs_resize_reclaim_nibble(
+			    nilfs, start, end, nfound, 0) < 0)
 			return -1;
 		retrycnt++;
 		nanosleep(&retry_interval, NULL);
-		goto retry;
 	}
-
-	if (retrycnt > 0 && pm_in_progress) {
-		nfound = nilfs_resize_count_inuse_segments(nilfs, start, end);
-		if (nfound >= 0) {
-			if (nfound > pm_max)
-				pm_max = nfound;
-			nilfs_resize_progress_update(pm_max - nfound);
-		}
-	}
-	return 0;
+	return retrycnt > 0;
 }
 
 static int nilfs_resize_reclaim_range(struct nilfs *nilfs, __u64 newnsegs)
 {
 	unsigned long long start, end, segnum;
+	ssize_t nfound;
 	int ret;
 
 	if (nilfs_resize_update_sustat(nilfs) < 0)
 		return -1;
 
 	if (newnsegs > sustat.ss_nsegs) {
-		myprintf("Error: Confused. Number of segments became larger "
-			 "than requested size.\n");
+		myprintf("Error: Confused. Number of segments became larger than requested size.\n");
 		return -1;
 	}
 	if (newnsegs == sustat.ss_nsegs)
@@ -805,6 +757,14 @@ static int nilfs_resize_reclaim_range(struct nilfs *nilfs, __u64 newnsegs)
 	ret = nilfs_resize_move_out_active_segments(nilfs, start, end);
 	if (ret < 0)
 		goto out;
+	if (ret && pm_in_progress) {
+		nfound = nilfs_resize_count_inuse_segments(nilfs, start, end);
+		if (nfound >= 0) {
+			if (nfound > pm_max)
+				pm_max = nfound;
+			nilfs_resize_progress_update(pm_max - nfound);
+		}
+	}
 
 	ret = -1;
 	segnum = start;
@@ -829,14 +789,14 @@ static int nilfs_resize_reclaim_range(struct nilfs *nilfs, __u64 newnsegs)
 		nmoved = nilfs_resize_move_segments(
 			nilfs, segnums, nfound, &reason);
 		if (nmoved < 0) {
-			myprintf("Error: operation failed during moving "
-				 "in-use segments: %s\n", strerror(errno));
+			myprintf("Error: operation failed during moving in-use segments: %s\n",
+				 strerror(errno));
 			goto out;
 		}
 
 		if (nmoved < nfound &&
 		    (reason & NILFS_RESIZE_SEGMENT_PROTECTED)) {
-			nilfs_resize_reclaim_nibble(nilfs, start, end, 2);
+			nilfs_resize_reclaim_nibble(nilfs, start, end, 2, 1);
 			if (pm_in_progress) {
 				nfound = nilfs_resize_count_inuse_segments(
 					nilfs, start, end);
@@ -858,9 +818,8 @@ static void nilfs_print_resize_error(int err, int shrink)
 {
 	myprintf("Error: failed to %s the filesystem: %s\n",
 		 shrink ? "shrink" : "extend", strerror(err));
-	if (err == ENOTTY) {
+	if (err == ENOTTY)
 		myprintf("       This kernel does not support the resize API.\n");
-	}
 }
 
 static int nilfs_resize_prompt(unsigned long long newsize)
@@ -898,15 +857,13 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 		goto out;
 
 	if (newnsegs < sustat.ss_nsegs) {
-		myprintf("%llu segments will be truncated from "
-			 "segnum %llu.\n",
+		myprintf("%llu segments will be truncated from segnum %llu.\n",
 			 (unsigned long long)sustat.ss_nsegs - newnsegs,
 			 (unsigned long long)newnsegs);
 	} else if (newnsegs == sustat.ss_nsegs) {
 		myprintf("No segments will be truncated.\n");
 	} else {
-		myprintf("Error: Confused. Number of segments "
-			 "became larger than requested size:\n"
+		myprintf("Error: Confused. Number of segments became larger than requested size:\n"
 			 "  old-nsegs=%llu new-nsegs=%llu\n",
 			 (unsigned long long)sustat.ss_nsegs,
 			 (unsigned long long)newnsegs);
@@ -919,10 +876,8 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 	if (nilfs_set_alloc_range(nilfs, 0, newsize) < 0) {
 		myprintf("Error: failed to limit allocation range: %s\n",
 			 strerror(errno));
-		if (errno == ENOTTY) {
-			myprintf("       This kernel does not support the "
-				 "set allocation range API.\n");
-		}
+		if (errno == ENOTTY)
+			myprintf("       This kernel does not support the set allocation range API.\n");
 		goto out;
 	}
 
@@ -949,9 +904,8 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 		if (nilfs_resize_lock_cleaner(nilfs, &sigset) < 0)
 			goto restore_alloc_range;
 
-		if (verbose) {
+		if (verbose)
 			myprintf("Truncating segments.\n");
-		}
 
 		ret = nilfs_resize(nilfs, newsize);
 		err = errno;
@@ -973,8 +927,7 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 			goto restore_alloc_range;
 
 		if (verbose) {
-			myprintf("In-use segment found again. "
-				 "will retry moving them.. (retry = %d)\n",
+			myprintf("In-use segment found again. will retry moving them.. (retry = %d)\n",
 				 retry + 1);
 		}
 	}
@@ -995,8 +948,8 @@ static int nilfs_extend_online(struct nilfs *nilfs, const char *device,
 	sigset_t sigset;
 
 	myprintf("Partition size = %llu bytes.\n"
-		 "Extend the filesystem size from %llu bytes to %llu "
-		 "bytes.\n", devsize, fs_devsize, newsize);
+		 "Extend the filesystem size from %llu bytes to %llu bytes.\n",
+		 devsize, fs_devsize, newsize);
 	if (!assume_yes && nilfs_resize_prompt(newsize) < 0)
 		goto out;
 
@@ -1005,6 +958,7 @@ static int nilfs_extend_online(struct nilfs *nilfs, const char *device,
 
 	if (nilfs_resize(nilfs, newsize) < 0) {
 		int err = errno;
+
 		nilfs_print_resize_error(err, 0);
 		goto out_unlock;
 	}
@@ -1020,10 +974,9 @@ static int nilfs_resize_online(const char *device, unsigned long long newsize)
 {
 	struct nilfs *nilfs;
 	struct nilfs_super_block *sb;
+	nilfs_cno_t cno;
 	int status = EXIT_FAILURE;
 
-	sync();
-
 	nilfs = nilfs_open(device, NULL,
 			   NILFS_OPEN_RAW | NILFS_OPEN_RDWR | NILFS_OPEN_GCLK);
 	if (nilfs == NULL) {
@@ -1042,6 +995,8 @@ static int nilfs_resize_online(const char *device, unsigned long long newsize)
 		goto out_unlock;
 	}
 
+	nilfs_sync(nilfs, &cno);
+
 	if (newsize > fs_devsize)
 		status = nilfs_extend_online(nilfs, device, newsize);
 	else
@@ -1109,12 +1064,24 @@ static int nilfs_resize_parse_size(const char *arg, unsigned long long *sizep)
 		;
 	} else if (endptr[1] == '\0') {
 		switch (endptr[0]) {
-		case 's': size <<= 9; break;
-		case 'K': size <<= 10; break;
-		case 'M': size <<= 20; break;
-		case 'G': size <<= 30; break;
-		case 'T': size <<= 40; break;
-		case 'P': size <<= 50; break;
+		case 's':
+			size <<= 9;
+			break;
+		case 'K':
+			size <<= 10;
+			break;
+		case 'M':
+			size <<= 20;
+			break;
+		case 'G':
+			size <<= 30;
+			break;
+		case 'T':
+			size <<= 40;
+			break;
+		case 'P':
+			size <<= 50;
+			break;
 		default:
 			return -1;
 		}
@@ -1149,15 +1116,14 @@ out:
 
 int main(int argc, char *argv[])
 {
+	char *last;
 	unsigned long long size;
 	struct stat statbuf;
 	char *device;
 	int status, ret;
 
-	if ((progname = strrchr(argv[0], '/')) != NULL)
-		progname++;
-	else
-		progname = argv[0];
+	last = strrchr(argv[0], '/');
+	progname = last ? last + 1 : argv[0];
 
 	nilfs_resize_parse_options(argc, argv);
 	if (show_version_only) {
@@ -1196,8 +1162,8 @@ int main(int argc, char *argv[])
 			goto out;
 		}
 		if (size > devsize) {
-			myprintf("Error: size larger than partition "
-				 "size (%llu bytes).\n", devsize);
+			myprintf("Error: size larger than partition size (%llu bytes).\n",
+				 devsize);
 			goto out;
 		}
 
@@ -1211,8 +1177,7 @@ int main(int argc, char *argv[])
 			unsigned long long size2;
 
 			size2 = size & ~(sector_size - 1);
-			myprintf("size %llu is not alinged to sector "
-				 "size. truncated to %llu.\n",
+			myprintf("size %llu is not aligned to sector size. truncated to %llu.\n",
 				 size, size2);
 			size = size2;
 		}
@@ -1222,8 +1187,7 @@ int main(int argc, char *argv[])
 
 
 	if (check_mount(device) == 0) {
-		myprintf("Error: %s is not currently mounted. Offline resising"
-			 "\n"
+		myprintf("Error: %s is not currently mounted. Offline resizing\n"
 			 "       is not supported at present.\n", device);
 		goto out;
 	} else {
